@@ -215,7 +215,7 @@ static void parseResult_FeedJob_(iFeedJob *d) {
                 entry->discovered = now;
                 entry->bookmarkId = d->bookmarkId;
                 setRange_String(&entry->url, url);
-                set_String(&entry->url, absoluteUrl_String(url_GmRequest(d->request), &entry->url));
+                set_String(&entry->url, canonicalUrl_String(absoluteUrl_String(url_GmRequest(d->request), &entry->url)));
                 setRange_String(&entry->title, title);
                 trimTitle_(&entry->title);
                 int year, month, day;
@@ -244,6 +244,7 @@ static void parseResult_FeedJob_(iFeedJob *d) {
                     set_String(&entry->url, &d->url);
                     appendChar_String(&entry->url, '#');
                     append_String(&entry->url, collect_String(urlEncode_String(title)));
+                    set_String(&entry->url, canonicalUrl_String(&entry->url));
                     delete_String(title);
                     pushBack_PtrArray(&d->results, entry);
                 }
@@ -498,8 +499,9 @@ static void load_Feeds_(iFeeds *d) {
                     break;
                 }
                 case 2: {
-                    /* TODO: All right, this could maybe use a bit more robust, structured
-                       format. The code below is messy. */
+                    /* TODO: Cleanup needed...
+                       All right, this could maybe use a bit more robust, structured format.
+                       The code below is messy. */
                     const uint32_t feedId = strtoul(line.start, NULL, 16);
                     if (!nextSplit_Rangecc(range_Block(src), "\n", &line)) {
                         goto aborted;
@@ -535,6 +537,7 @@ static void load_Feeds_(iFeeds *d) {
                         entry->discovered.ts.tv_sec = discovered;
                         set_String(&entry->url, url);
                         stripDefaultUrlPort_String(&entry->url);
+                        set_String(&entry->url, canonicalUrl_String(&entry->url));
                         set_String(&entry->title, title);
                         insert_SortedArray(&d->entries, &entry);
                     }
